library jaguar_auth.session;

import 'dart:io';
import 'dart:async';
import 'dart:convert';
import 'package:jaguar/jaguar.dart';
import 'package:auth_header/auth_header.dart';
import 'package:jaguar_jwt/jaguar_jwt.dart';

part 'session_jwt_cookie.dart';
part 'session_jwt_auth_header.dart';
part 'session_cookie.dart';

/// Session management interface
abstract class SessionManager {
  /// Returns a session value by given [key]
  String getInValue(String key);

  /// Update session info
  Future<Null> updateSession(Map<String, String> values);

  /// Clears the session
  Future<Null> clearSession();

  /// Parses session from the given [request]
  Future<Null> parseRequest(Request request);

  /// Updates [response] with session details
  Future<Response> updateResponse(Request request, Response resp);

  /// Validates the current session
  ///
  /// Must throw exception on validation failure
  Future<Null> validate();
}

/// Wrapper to wrap routes with [SessionInterceptor]
class WrapSessionInterceptor implements RouteWrapper<SessionInterceptor> {
  /// Session manager used by [SessionInterceptor] to parse and update session
  /// to [HttpRequest]
  final SessionManager sessionManager;

  /// Unique ID of the Interceptor instance on a route
  ///
  /// It is used to identify an interceptor on a route chain
  final String id;

  /// Offers a way to make non-constant wrapper parameters
  final Map<Symbol, MakeParam> makeParams;

  const WrapSessionInterceptor({this.sessionManager, this.id, this.makeParams});

  /// Creates [SessionInterceptor] instance
  SessionInterceptor createInterceptor() =>
      new SessionInterceptor(this.sessionManager);
}

/// Interceptor to parse and update session to [HttpRequest]
///
/// It allows offers an instance of [SessionManager] as input to other interceptors
/// and routes
class SessionInterceptor extends Interceptor {
  /// Session manager to parse and update session on [HttpRequest]
  ///
  /// Offered as [Input] to other interceptors and routes
  final SessionManager sessionManager;

  SessionInterceptor(this.sessionManager);

  Future<SessionManager> pre(Request req) async {
    await sessionManager.parseRequest(req);
    return sessionManager;
  }

  @InputRouteResponse()
  Future<Response> post(Request req, Response resp) async {
    return await sessionManager.updateResponse(req, resp);
  }
}
