part of jaguar_auth.session;

/// Session manager based on JWT with cookie as transport mechanism
///
/// Stores all session data in JWT token
class JwtCookieSessionManager implements SessionManager {
  /// Name of the cookie on which session data is stored
  final String cookieName;

  /// JWT authorization context
  final JwtAuthorizationContext jwtAuthorizationContext;

  /// JWT authentication context
  final JwtAuthenticationContext jwtAuthenticationContext;

  JaguarJwtClaimSet _inClaimSet;

  /// Incoming session data parsed from the request
  final Map<String, String> _inValues = {};

  /// Outgoing session data that must be written to the response
  final Map<String, String> _outValues = {};

  bool _shouldUpdate = false;

  JwtCookieSessionManager(
      this.jwtAuthenticationContext, this.jwtAuthorizationContext,
      [this.cookieName = 'session']);

  /// Returns a session value by given [key]
  String getInValue(String key) => _inValues[key];

  /// Update session info
  Future<Null> updateSession(Map<String, String> values) async {
    _outValues.addAll(values);
    _shouldUpdate = true;
  }

  /// Clears the session
  Future<Null> clearSession() async {
    _outValues.clear();
    _shouldUpdate = true;
  }

  /// Parses session from the given [request]
  Future<Null> parseRequest(Request request) async {
    for (Cookie cook in request.cookies) {
      if (cook.name == cookieName) {
        dynamic valueMap = _decode(cook.value);
        if (valueMap is Map<String, String>) {
          _inValues.addAll(valueMap);
        }
        break;
      }
    }
  }

  /// Validates the current session
  ///
  /// Must throw exception on validation failure
  Future<Null> validate() async {
    if (_inClaimSet == null) {
      throw new UnAuthorizedError();
    }

    //Validate token
    if (!jwtAuthorizationContext.isClaimSetValid(_inClaimSet)) {
      throw new UnAuthorizedError();
    }
  }

  String get _subject => _outValues['sub'];

  /// Updates [response] with session details
  Future<Response> updateResponse(Request request, Response resp) async {
    if (_shouldUpdate) {
      Cookie cook = new Cookie(cookieName, _encode());
      resp.cookies.add(cook);
    }
    return resp;
  }

  String _encode() {
    String token = jwtAuthenticationContext.encodeForSubject(_subject,
        payload: _outValues);
    return const Base64Codec.urlSafe().encode(token.codeUnits);
  }

  dynamic _decode(String data) {
    String tokenStr =
        new String.fromCharCodes(const Base64Codec.urlSafe().decode(data));

    //Decode and provide token
    _inClaimSet = jwtAuthorizationContext.decodeToken(tokenStr).claimSet;
    Map ret = _inClaimSet.payload ?? {};
    ret['sub'] = _inClaimSet.subject;

    return ret;
  }
}
